@@ -29,7 +29,9 @@
 
 
 #include "glheader.h"
+#if FEATURE_colortable
 #include "colortab.h"
+#endif
 #include "context.h"
 #include "enums.h"
 #include "fbobject.h"
@@ -187,7 +189,9 @@ _mesa_delete_texture_object( GLcontext *ctx, struct gl_texture_object *texObj )
     */
    texObj->Target = 0x99;
 
+#if FEATURE_colortable
    _mesa_free_colortable_data(&texObj->Palette);
+#endif
 
    /* free the texture images */
    for (face = 0; face < 6; face++) {
@@ -379,6 +383,18 @@ _mesa_test_texobj_completeness( const GLcontext *ctx,
 
    t->_Complete = GL_TRUE;  /* be optimistic */
 
+   /* Detect cases where the application set the base level to an invalid
+    * value.
+    */
+   if ((baseLevel < 0) || (baseLevel > MAX_TEXTURE_LEVELS)) {
+      char s[100];
+      _mesa_sprintf(s, "obj %p (%d) base level = %d is invalid",
+              (void *) t, t->Name, baseLevel);
+      incomplete(t, s);
+      t->_Complete = GL_FALSE;
+      return;
+   }
+
    /* Always need the base level image */
    if (!t->Image[0][baseLevel]) {
       char s[100];
@@ -746,24 +762,31 @@ unbind_texobj_from_texunits(GLcontext *ctx, struct gl_texture_object *texObj)
       struct gl_texture_unit *unit = &ctx->Texture.Unit[u];
       if (texObj == unit->Current1D) {
          _mesa_reference_texobj(&unit->Current1D, ctx->Shared->Default1D);
+         ASSERT(unit->Current1D);
       }
       else if (texObj == unit->Current2D) {
          _mesa_reference_texobj(&unit->Current2D, ctx->Shared->Default2D);
+         ASSERT(unit->Current2D);
       }
       else if (texObj == unit->Current3D) {
          _mesa_reference_texobj(&unit->Current3D, ctx->Shared->Default3D);
+         ASSERT(unit->Current3D);
       }
       else if (texObj == unit->CurrentCubeMap) {
          _mesa_reference_texobj(&unit->CurrentCubeMap, ctx->Shared->DefaultCubeMap);
+         ASSERT(unit->CurrentCubeMap);
       }
       else if (texObj == unit->CurrentRect) {
          _mesa_reference_texobj(&unit->CurrentRect, ctx->Shared->DefaultRect);
+         ASSERT(unit->CurrentRect);
       }
       else if (texObj == unit->Current1DArray) {
          _mesa_reference_texobj(&unit->Current1DArray, ctx->Shared->Default1DArray);
+         ASSERT(unit->Current1DArray);
       }
       else if (texObj == unit->Current2DArray) {
          _mesa_reference_texobj(&unit->Current2DArray, ctx->Shared->Default2DArray);
+         ASSERT(unit->Current2DArray);
       }
    }
 }
@@ -937,24 +960,31 @@ _mesa_BindTexture( GLenum target, GLuint texName )
    switch (target) {
       case GL_TEXTURE_1D:
          _mesa_reference_texobj(&texUnit->Current1D, newTexObj);
+         ASSERT(texUnit->Current1D);
          break;
       case GL_TEXTURE_2D:
          _mesa_reference_texobj(&texUnit->Current2D, newTexObj);
+         ASSERT(texUnit->Current2D);
          break;
       case GL_TEXTURE_3D:
          _mesa_reference_texobj(&texUnit->Current3D, newTexObj);
+         ASSERT(texUnit->Current3D);
          break;
       case GL_TEXTURE_CUBE_MAP_ARB:
          _mesa_reference_texobj(&texUnit->CurrentCubeMap, newTexObj);
+         ASSERT(texUnit->CurrentCubeMap);
          break;
       case GL_TEXTURE_RECTANGLE_NV:
          _mesa_reference_texobj(&texUnit->CurrentRect, newTexObj);
+         ASSERT(texUnit->CurrentRect);
          break;
       case GL_TEXTURE_1D_ARRAY_EXT:
          texUnit->Current1DArray = newTexObj;
+         ASSERT(texUnit->Current1DArray);
          break;
       case GL_TEXTURE_2D_ARRAY_EXT:
          texUnit->Current2DArray = newTexObj;
+         ASSERT(texUnit->Current2DArray);
          break;
       default:
          /* Bad target should be caught above */
